/* 
   Project: FísicaLab

   Copyright (C) 2009, 2010, 2015 Free Software Foundation

   Author: German A. Arias <germanandre@gmx.es>
   
   Created: 2008-09-10 18:56:00 -0600 by german

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 3 of the License, or (at your option) any later version.
 
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
 
   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#import <AppKit/AppKit.h>
#import "Renaissance/Renaissance.h"
#import "AppController.h"

int (*linkRenaissanceIn)(int, const char **) = GSMarkupApplicationMain; 

int 
main(int argc, const char *argv[])
{
  CREATE_AUTORELEASE_POOL (pool);
  [NSApplication sharedApplication];
  [NSApp setDelegate: [AppController new]];
  [NSBundle loadGSMarkupNamed: @"menu" owner: [NSApp delegate]];
  [NSApp run];
  RELEASE(pool);

  return 0;
}

